@@ -40,6 +40,7 @@
 
 #include <glib.h>
 #include <glib/gi18n-lib.h>
+#include <gdk/gdk.h>
 
 #include "burn-basics.h"
 
@@ -252,7 +253,7 @@ brasero_medium_support_flags (BraseroMedium *self,
  *
  **/
 gchar *
-brasero_medium_get_tooltip (BraseroMedium *self)
+brasero_medium_get_tooltip (BraseroMedium *medium)
 {
 	BraseroMediumPrivate *priv;
 	BraseroDrive *drive;
@@ -261,12 +262,12 @@ brasero_medium_get_tooltip (BraseroMedium *self)
 	gchar *label;
 	gchar *name;
 
-	g_return_val_if_fail (self != NULL, NULL);
-	g_return_val_if_fail (BRASERO_IS_MEDIUM (self), NULL);
+	g_return_val_if_fail (medium != NULL, NULL);
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), NULL);
 
-	priv = BRASERO_MEDIUM_PRIVATE (self);
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
-	media = brasero_medium_get_status (BRASERO_MEDIUM (self));
+	media = brasero_medium_get_status (BRASERO_MEDIUM (medium));
 	if (media & BRASERO_MEDIUM_FILE) {
 		/* Translators: This is a fake drive, a file, and means that
 		 * when we're writing, we're writing to a file and create an
@@ -274,8 +275,8 @@ brasero_medium_get_tooltip (BraseroMedium *self)
 		return g_strdup (_("Image File"));
 	}
 
-	type = brasero_medium_get_type_string (BRASERO_MEDIUM (self));
-	drive = brasero_medium_get_drive (BRASERO_MEDIUM (self));
+	type = brasero_medium_get_type_string (BRASERO_MEDIUM (medium));
+	drive = brasero_medium_get_drive (BRASERO_MEDIUM (medium));
 	name = brasero_drive_get_display_name (drive);
 
 	if (media & BRASERO_MEDIUM_BLANK) {
@@ -332,6 +333,9 @@ brasero_medium_get_type_string (BraseroMedium *medium)
 {
 	BraseroMediumPrivate *priv;
 
+	g_return_val_if_fail (medium != NULL, NULL);
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), NULL);
+
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 	return priv->type;
 }
@@ -353,6 +357,8 @@ brasero_medium_get_status (BraseroMedium *medium)
 	if (!medium)
 		return BRASERO_MEDIUM_NONE;
 
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), BRASERO_MEDIUM_NONE);
+
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 	return priv->info;
 }
@@ -377,6 +383,9 @@ brasero_medium_get_last_data_track_address (BraseroMedium *medium,
 	BraseroMediumPrivate *priv;
 	BraseroMediumTrack *track = NULL;
 
+	g_return_val_if_fail (medium != NULL, FALSE);
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), FALSE);
+
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
 	for (iter = priv->tracks; iter; iter = iter->next) {
@@ -423,6 +432,9 @@ brasero_medium_get_last_data_track_space (BraseroMedium *medium,
 	BraseroMediumPrivate *priv;
 	BraseroMediumTrack *track = NULL;
 
+	g_return_val_if_fail (medium != NULL, FALSE);
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), FALSE);
+
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
 	for (iter = priv->tracks; iter; iter = iter->next) {
@@ -461,10 +473,13 @@ brasero_medium_get_last_data_track_space (BraseroMedium *medium,
 guint
 brasero_medium_get_track_num (BraseroMedium *medium)
 {
-	guint retval = 0;
 	GSList *iter;
+	guint retval = 0;
 	BraseroMediumPrivate *priv;
 
+	g_return_val_if_fail (medium != NULL, 0);
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), 0);
+
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 	for (iter = priv->tracks; iter; iter = iter->next) {
 		BraseroMediumTrack *current;
@@ -528,6 +543,9 @@ brasero_medium_get_track_space (BraseroMedium *medium,
 	BraseroMediumPrivate *priv;
 	BraseroMediumTrack *track;
 
+	g_return_val_if_fail (medium != NULL, FALSE);
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), FALSE);
+
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
 	track = brasero_medium_get_track (medium, num);
@@ -570,6 +588,9 @@ brasero_medium_get_track_address (BraseroMedium *medium,
 	BraseroMediumPrivate *priv;
 	BraseroMediumTrack *track;
 
+	g_return_val_if_fail (medium != NULL, FALSE);
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), FALSE);
+
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
 	track = brasero_medium_get_track (medium, num);
@@ -603,6 +624,9 @@ brasero_medium_get_next_writable_address (BraseroMedium *medium)
 {
 	BraseroMediumPrivate *priv;
 
+	g_return_val_if_fail (medium != NULL, 0);
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), 0);
+
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
 	/* There is one exception to this with closed DVD+RW/DVD-RW restricted */
@@ -640,6 +664,9 @@ brasero_medium_get_max_write_speed (BraseroMedium *medium)
 {
 	BraseroMediumPrivate *priv;
 
+	g_return_val_if_fail (medium != NULL, 0);
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), 0);
+
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 	return priv->max_wrt * 1000;
 }
@@ -662,6 +689,9 @@ brasero_medium_get_write_speeds (BraseroMedium *medium)
 	guint max = 0;
 	guint i;
 
+	g_return_val_if_fail (medium != NULL, NULL);
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), NULL);
+
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
 	if (!priv->wr_speeds)
@@ -721,6 +751,9 @@ brasero_medium_get_data_size (BraseroMedium *medium,
 	BraseroMediumPrivate *priv;
 	BraseroMediumTrack *track = NULL;
 
+	g_return_if_fail (medium != NULL);
+	g_return_if_fail (BRASERO_IS_MEDIUM (medium));
+
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
 	if (!priv->tracks) {
@@ -770,6 +803,9 @@ brasero_medium_get_free_space (BraseroMedium *medium,
 	BraseroMediumPrivate *priv;
 	BraseroMediumTrack *track = NULL;
 
+	g_return_if_fail (medium != NULL);
+	g_return_if_fail (BRASERO_IS_MEDIUM (medium));
+
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
 	if (!priv->tracks) {
@@ -848,6 +884,9 @@ brasero_medium_get_capacity (BraseroMedium *medium,
 {
 	BraseroMediumPrivate *priv;
 
+	g_return_if_fail (medium != NULL);
+	g_return_if_fail (BRASERO_IS_MEDIUM (medium));
+
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
 	if (priv->info & BRASERO_MEDIUM_REWRITABLE) {
@@ -1223,7 +1262,7 @@ brasero_medium_get_capacity_by_type (BraseroMedium *self,
 
 	if (priv->info & BRASERO_MEDIUM_CD)
 		brasero_medium_get_capacity_CD_RW (self, handle, code);
-	else
+	else	/* Works for BD-RE as well */
 		brasero_medium_get_capacity_DVD_RW (self, handle, code);
 
 	return BRASERO_BURN_OK;
@@ -1287,6 +1326,8 @@ brasero_medium_get_speed_mmc3 (BraseroMedium *self,
 	priv->max_rd = max_rd;
 	priv->max_wrt = max_wrt;
 
+	BRASERO_MEDIA_LOG ("Maximum Speed (mmc3) %i", max_wrt);
+
 end:
 
 	g_free (wrt_perf);
@@ -1332,14 +1373,29 @@ brasero_medium_get_page_2A_write_speed_desc (BraseroMedium *self,
 
 	page_2A = (BraseroScsiStatusPage *) &data->page;
 
-	/* FIXME: the following is not necessarily true */
-	if (size < sizeof (BraseroScsiStatusPage)) {
+	if (size < 18) {
 		g_free (data);
-
-		BRASERO_MEDIA_LOG ("wrong size in page");
+		BRASERO_MEDIA_LOG ("wrong page size");
 		return BRASERO_BURN_ERR;
 	}
 
+	priv->max_rd = BRASERO_GET_16 (page_2A->rd_max_speed);
+	priv->max_wrt = BRASERO_GET_16 (page_2A->wr_max_speed);
+
+	/* Check if we can use the speed descriptors; if not use maximum */
+	if (size < 20) {
+		BRASERO_MEDIA_LOG ("Maximum Speed (Page 2A [old]) %i", priv->max_wrt);
+
+		/* also add fake speed descriptors */
+		priv->wr_speeds = g_new0 (gint, 2);
+		priv->wr_speeds [0] = BRASERO_GET_16 (page_2A->wr_max_speed);
+		priv->rd_speeds = g_new0 (gint, 2);
+		priv->rd_speeds [0] = BRASERO_GET_16 (page_2A->rd_max_speed);
+
+		g_free (data);
+		return BRASERO_BURN_OK;
+	}
+
 	desc_num = BRASERO_GET_16 (page_2A->wr_speed_desc_num);
 	max_num = size -
 		  sizeof (BraseroScsiStatusPage) -
@@ -1359,66 +1415,33 @@ brasero_medium_get_page_2A_write_speed_desc (BraseroMedium *self,
 		max_wrt = MAX (max_wrt, priv->wr_speeds [i]);
 	}
 
-	if (!max_wrt)
-		priv->max_wrt = BRASERO_GET_16 (page_2A->wr_max_speed);
-	else
+	if (max_wrt)
 		priv->max_wrt = max_wrt;
 
-	priv->max_rd = BRASERO_GET_16 (page_2A->rd_max_speed);
+	BRASERO_MEDIA_LOG ("Maximum Speed (Page 2A) %i", priv->max_wrt);
 	g_free (data);
 
 	return BRASERO_BURN_OK;
 }
 
 static BraseroBurnResult
-brasero_medium_get_page_2A_max_speed (BraseroMedium *self,
-				      BraseroDeviceHandle *handle,
-				      BraseroScsiErrCode *code)
+brasero_medium_get_speed (BraseroMedium *self,
+			  BraseroDeviceHandle *handle,
+			  BraseroScsiErrCode *code)
 {
-	BraseroScsiStatusPage *page_2A = NULL;
-	BraseroScsiModeData *data = NULL;
-	BraseroMediumPrivate *priv;
 	BraseroScsiResult result;
-	int size = 0;
 
-	BRASERO_MEDIA_LOG ("Retrieving speed (2A max)");
+	BRASERO_MEDIA_LOG ("Retrieving media available speeds");
 
-	priv = BRASERO_MEDIUM_PRIVATE (self);
+	result = brasero_medium_get_speed_mmc3 (self, handle, code);
+	if (result == BRASERO_BURN_OK)
+		return result;
 
-	result = brasero_spc1_mode_sense_get_page (handle,
-						   BRASERO_SPC_PAGE_STATUS,
-						   &data,
-						   &size,
-						   code);
-	if (result != BRASERO_SCSI_OK) {
-		g_free (data);
-
-		BRASERO_MEDIA_LOG ("MODE SENSE failed");
-		return BRASERO_BURN_ERR;
-	}
-
-	page_2A = (BraseroScsiStatusPage *) &data->page;
-
-	if (size < 0x14) {
-		g_free (data);
-
-		BRASERO_MEDIA_LOG ("wrong page size");
-		return BRASERO_BURN_ERR;
-	}
-
-	priv->max_rd = BRASERO_GET_16 (page_2A->rd_max_speed);
-	priv->max_wrt = BRASERO_GET_16 (page_2A->wr_max_speed);
-
-	/* also add it to the speed descriptors */
-	priv->wr_speeds = g_new0 (gint, 2);
-	priv->wr_speeds [0] = BRASERO_GET_16 (page_2A->wr_max_speed);
-	priv->rd_speeds = g_new0 (gint, 2);
-	priv->rd_speeds [0] = BRASERO_GET_16 (page_2A->rd_max_speed);
-
-	g_free (data);
-	return BRASERO_BURN_OK;
+	/* Fallback */
+	result = brasero_medium_get_page_2A_write_speed_desc (self, handle, code);
+	return result;
 }
- 
+
 /**
  * Functions to get information about disc contents
  */
@@ -1716,6 +1739,7 @@ brasero_medium_track_set_leadout (BraseroMedium *self,
 		return BRASERO_BURN_OK;
 	}
 
+	/* NOTE this works for CDR, DVDR+-, BDR-SRM */
 	/* make sure the current write mode is TAO. Otherwise the drive will
 	 * return the first sector of the pregap instead of the first user
 	 * accessible sector. */
@@ -1743,8 +1767,15 @@ brasero_medium_track_set_leadout (BraseroMedium *self,
 		wrt_page->session_format = 0;
 		BRASERO_SET_16 (wrt_page->pause_len, 150);
 
-		wrt_page->write_type = BRASERO_SCSI_WRITE_TAO;
-		wrt_page->track_mode = 4; /* should be 5 for DVD-R(W) */
+		if (priv->info & BRASERO_MEDIUM_CD) {
+			wrt_page->write_type = BRASERO_SCSI_WRITE_TAO;
+			wrt_page->track_mode = 4;
+		}
+		else if (priv->info & BRASERO_MEDIUM_DVD) {
+			wrt_page->write_type = BRASERO_SCSI_WRITE_PACKET_INC;
+			wrt_page->track_mode = 5;
+		}
+
 		wrt_page->data_block_type = 8;
 
 		result = brasero_spc1_mode_select (handle, data, size, code);
@@ -1805,7 +1836,6 @@ brasero_medium_track_set_leadout (BraseroMedium *self,
 					       &track_info,
 					       &size,
 					       code);
-
 	if (result != BRASERO_SCSI_OK) {
 		/* This only for CD-R */
 		if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_CDR|BRASERO_MEDIUM_BLANK))
@@ -2072,6 +2102,42 @@ brasero_medium_get_DVD_id (BraseroMedium *self,
 	g_free (hdr);
 }
 
+static BraseroBurnResult
+brasero_medium_set_blank (BraseroMedium *self,
+			  BraseroDeviceHandle *handle,
+			  gint first_open_track,
+			  BraseroScsiErrCode *code)
+{
+	BraseroMediumPrivate *priv;
+	BraseroMediumTrack *track;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	BRASERO_MEDIA_LOG ("Empty media");
+
+	priv->info |= BRASERO_MEDIUM_BLANK;
+	priv->block_size = 2048;
+
+	priv->first_open_track = first_open_track;
+	BRASERO_MEDIA_LOG ("First open track %d", priv->first_open_track);
+
+	if (BRASERO_MEDIUM_RANDOM_WRITABLE (priv->info))
+		brasero_medium_add_DVD_plus_RW_leadout (self);
+	else {
+		track = g_new0 (BraseroMediumTrack, 1);
+		track->start = 0;
+		track->type = BRASERO_MEDIUM_TRACK_LEADOUT;
+		priv->tracks = g_slist_prepend (priv->tracks, track);
+			
+		brasero_medium_track_set_leadout (self,
+						  handle,
+						  track,
+						  code);
+	}
+
+	return BRASERO_BURN_OK;
+}
+
 static BraseroBurnResult
 brasero_medium_get_contents (BraseroMedium *self,
 			     BraseroDeviceHandle *handle,
@@ -2111,38 +2177,30 @@ brasero_medium_get_contents (BraseroMedium *self,
 	priv->first_open_track = -1;
 
 	if (info->status == BRASERO_SCSI_DISC_EMPTY) {
-		BraseroMediumTrack *track;
-
-		BRASERO_MEDIA_LOG ("Empty media");
-
-		priv->info |= BRASERO_MEDIUM_BLANK;
-		priv->block_size = 2048;
-
-		priv->first_open_track = BRASERO_FIRST_TRACK_IN_LAST_SESSION (info);
-		BRASERO_MEDIA_LOG ("First open track %d", priv->first_open_track);
-
-		if (BRASERO_MEDIUM_RANDOM_WRITABLE (priv->info))
-			brasero_medium_add_DVD_plus_RW_leadout (self);
-		else {
-			track = g_new0 (BraseroMediumTrack, 1);
-			track->start = 0;
-			track->type = BRASERO_MEDIUM_TRACK_LEADOUT;
-			priv->tracks = g_slist_prepend (priv->tracks, track);
-			
-			brasero_medium_track_set_leadout (self,
-							  handle,
-							  track,
-							  code);
-		}
+		brasero_medium_set_blank (self,
+					  handle,
+					  BRASERO_FIRST_TRACK_IN_LAST_SESSION (info),
+					  code);
 	}
 	else if (info->status == BRASERO_SCSI_DISC_INCOMPLETE) {
-		priv->info |= BRASERO_MEDIUM_APPENDABLE;
-		BRASERO_MEDIA_LOG ("Appendable media");
+		if (!BRASERO_MEDIUM_RANDOM_WRITABLE (priv->info)) {
+			priv->info |= BRASERO_MEDIUM_APPENDABLE;
+			BRASERO_MEDIA_LOG ("Appendable media");
 
-		priv->first_open_track = BRASERO_FIRST_TRACK_IN_LAST_SESSION (info);
-		BRASERO_MEDIA_LOG ("First track in last open session %d", priv->first_open_track);
+			priv->first_open_track = BRASERO_FIRST_TRACK_IN_LAST_SESSION (info);
+			BRASERO_MEDIA_LOG ("First track in last open session %d", priv->first_open_track);
 
-		result = brasero_medium_get_sessions_info (self, handle, code);
+			result = brasero_medium_get_sessions_info (self, handle, code);
+		}
+		else {
+			/* if that type of media is in incomplete state that
+			 * means it has just been formatted. And therefore it's
+			 * blank. */
+			brasero_medium_set_blank (self,
+						  handle,
+						  BRASERO_FIRST_TRACK_IN_LAST_SESSION (info),
+						  code);
+		}
 	}
 	else if (info->status == BRASERO_SCSI_DISC_FINALIZED) {
 		priv->info |= BRASERO_MEDIUM_CLOSED;
@@ -2155,457 +2213,6 @@ brasero_medium_get_contents (BraseroMedium *self,
 	return result;
 }
 
-/**
- * These are special routines for old CD-R(W) drives that don't conform to MMC
- */
-
-static void
-brasero_medium_set_track_type (BraseroMedium *self,
-			       BraseroMediumTrack *track,
-			       guchar control)
-{
-	BraseroMediumPrivate *priv;
-
-	priv = BRASERO_MEDIUM_PRIVATE (self);
-
-	if (control & BRASERO_SCSI_TRACK_COPY)
-		track->type |= BRASERO_MEDIUM_TRACK_COPY;
-
-	if (!(control & BRASERO_SCSI_TRACK_DATA)) {
-		track->type |= BRASERO_MEDIUM_TRACK_AUDIO;
-		priv->info |= BRASERO_MEDIUM_HAS_AUDIO;
-
-		if (control & BRASERO_SCSI_TRACK_PREEMP)
-			track->type |= BRASERO_MEDIUM_TRACK_PREEMP;
-
-		if (control & BRASERO_SCSI_TRACK_4_CHANNELS)
-			track->type |= BRASERO_MEDIUM_TRACK_4_CHANNELS;
-	}
-	else {
-		track->type |= BRASERO_MEDIUM_TRACK_DATA;
-		priv->info |= BRASERO_MEDIUM_HAS_DATA;
-
-		if (control & BRASERO_SCSI_TRACK_DATA_INCREMENTAL)
-			track->type |= BRASERO_MEDIUM_TRACK_INCREMENTAL;
-	}
-}
-
-/**
- * return :
- *  0 when it's not possible to determine (fallback to formatted toc)
- * -1 for BCD
- *  1 for HEX */
-static guint
-brasero_medium_check_BCD_use (BraseroMedium *self,
-			      BraseroDeviceHandle *handle,
-			      BraseroScsiRawTocDesc *desc,
-			      guint num,
-			      BraseroScsiErrCode *code)
-{
-	guint i;
-	int size;
-	guint leadout = 0;
-	guint track_num = 0;
-	gboolean use_BCD = TRUE;
-	gboolean use_HEX = TRUE;
-	BraseroScsiResult result;
-	BraseroScsiTrackInfo track_info;
-	guint start_BCD, start_LBA, track_start;
-
-	/* first check if all values are valid BCD numbers in the descriptors */
-	for (i = 0; i < num; i++) {
-		if (desc [i].adr == 1 && desc [i].point <= BRASERO_SCSI_Q_SUB_CHANNEL_TRACK_START) {
-			if (!BRASERO_IS_BCD_VALID (desc [i].p_min)
-			||  !BRASERO_IS_BCD_VALID (desc [i].p_sec)
-			||  !BRASERO_IS_BCD_VALID (desc [i].p_frame)) {
-				use_BCD = FALSE;
-				break;
-			}
-		}
-		else if (desc [i].point == BRASERO_SCSI_Q_SUB_CHANNEL_LEADOUT_START) {
-			if (!BRASERO_IS_BCD_VALID (desc [i].p_min)
-			||  !BRASERO_IS_BCD_VALID (desc [i].p_sec)
-			||  !BRASERO_IS_BCD_VALID (desc [i].p_frame)) {
-				use_BCD = FALSE;
-				break;
-			}
-		}
-	}
-
-	/* then check if there are valid Hex values */
-	for (i = 0; i < num; i++) {
-		if (desc [i].adr != 1 || desc [i].point > BRASERO_SCSI_Q_SUB_CHANNEL_TRACK_START)
-			continue;
-
-		if (desc [i].p_min > 99
-		||  desc [i].p_sec > 59
-		||  desc [i].p_frame > 74) {
-			use_HEX = FALSE;
-			break;
-		}
-	}
-
-	if (use_BCD != use_HEX) {
-		if (use_BCD)
-			return -1;
-
-		return 1;
-	}
-
-	/* To check if the drive uses BCD values or HEX values we ask for the 
-	 * track information that contains also the start for the track but in
-	 * HEX values. If values are the same then it works. */
-
-	/* NOTE: there could be another way to do it: get first track, in LBA
-	 * and BCD it must be 150. */
-
-	/* First find the first track and get track start address in BCD */
-	BRASERO_MEDIA_LOG ("Retrieving track information to determine number format");
-
-	for (i = 0; i < num; i++) {
-		if (desc [i].adr == BRASERO_SCSI_Q_SUB_CHANNEL_LEADIN_MODE5
-		&&  desc [i].point == BRASERO_SCSI_Q_SUB_CHANNEL_MULTI_NEXT_SESSION) {
-			/* store the leadout number just in case */
-			leadout = i;
-			continue;
-		}
-
-		if (desc [i].adr != 1 || desc [i].point > BRASERO_SCSI_Q_SUB_CHANNEL_TRACK_START)
-			continue;
-
-		track_num ++;
-
-		start_BCD = BRASERO_MSF_TO_LBA (BRASERO_GET_BCD (desc [i].p_min),
-						BRASERO_GET_BCD (desc [i].p_sec),
-						BRASERO_GET_BCD (desc [i].p_frame));
-
-		start_LBA = BRASERO_MSF_TO_LBA (desc [i].p_min,
-						desc [i].p_sec,
-						desc [i].p_frame);
-
-		BRASERO_MEDIA_LOG ("Comparing to track information from READ TRACK INFO for track %i", track_num);
-
-		size = 36;
-		start_LBA -= 150;
-		start_BCD -= 150;
-
-/*
-		result = brasero_mmc1_read_track_info (handle,
-						       track_num,
-						       &track_info,
-						       &size,
-						       code);
-
-		if (result != BRASERO_SCSI_OK) {
-			BRASERO_MEDIA_LOG ("READ TRACK INFO failed");
-*/			/* Fallback to formatted toc */
-/*			return 0;
-		}
-*/
-		track_start = BRASERO_GET_32 (track_info.start_lba);
-		BRASERO_MEDIA_LOG ("comparing DCB %i and LBA %i to real start address %i",
-				  start_BCD, start_LBA, track_start);
-
-		/* try to find a conclusive match */
-		if (track_start == start_BCD && track_start != start_LBA)
-			return -1;
-
-		if (track_start == start_LBA && track_start != start_BCD)
-			return 1;
-	}
-
-	/* Our last chance, the leadout.
-	 * NOTE: no need to remove 150 sectors here. */
-	start_BCD = BRASERO_MSF_TO_LBA (BRASERO_GET_BCD (desc [leadout].min),
-					BRASERO_GET_BCD (desc [leadout].sec),
-					BRASERO_GET_BCD (desc [leadout].frame));
-
-	start_LBA = BRASERO_MSF_TO_LBA (desc [leadout].min,
-					desc [leadout].sec,
-					desc [leadout].frame);
-
-	BRASERO_MEDIA_LOG ("Comparing to track information from READ TRACK INFO for leadout");
-
-	size = 36;
-
-	/* leadout number is number of tracks + 1 */
-	result = brasero_mmc1_read_track_info (handle,
-					       track_num + 1,
-					       &track_info,
-					       &size,
-					       code);
-
-	if (result != BRASERO_SCSI_OK) {
-		BRASERO_MEDIA_LOG ("READ TRACK INFO failed for leadout");
-		/* Fallback to formatted toc */
-		return 0;
-	}
-
-	track_start = BRASERO_GET_32 (track_info.start_lba);
-	BRASERO_MEDIA_LOG ("comparing DCB %i and LBA %i to real start address %i",
-			  start_BCD, start_LBA, track_start);
-
-	/* try to find a conclusive match */
-	if (track_start == start_BCD && track_start != start_LBA)
-		return -1;
-
-	if (track_start == start_LBA && track_start != start_BCD)
-		return 1;
-
-	/* fallback to formatted toc */
-	return 0;
-}
-
-/**
- * The reason why we use this perhaps more lengthy method is that with
- * multisession discs, the first track is reported to be two sectors shorter
- * than it should. As I don't know why and since the following works we use
- * this one. */
-static BraseroBurnResult
-brasero_medium_get_CD_sessions_info (BraseroMedium *self,
-				     BraseroDeviceHandle *handle,
-				     BraseroScsiErrCode *code)
-{
-	gint use_bcd;
-	GSList *iter;
-	int num, i, size;
-	gint leadout_start = 0;
-	BraseroScsiResult result;
-	BraseroMediumPrivate *priv;
-	BraseroScsiRawTocDesc *desc;
-	BraseroScsiRawTocData *toc = NULL;
-
-	BRASERO_MEDIA_LOG ("Reading Raw Toc");
-
-	priv = BRASERO_MEDIUM_PRIVATE (self);
-
-	size = 0;
-	result = brasero_mmc1_read_toc_raw (handle,
-					    0,
-					    &toc,
-					    &size,
-					    code);
-	if (result != BRASERO_SCSI_OK) {
-		BRASERO_MEDIA_LOG ("READ TOC failed");
-		return BRASERO_BURN_ERR;
-	}
-
-	num = (size - sizeof (BraseroScsiRawTocData)) /
-	       sizeof (BraseroScsiRawTocDesc);
-
-	BRASERO_MEDIA_LOG ("%i track(s) found", num);
-
-	desc = toc->desc;
-	use_bcd = brasero_medium_check_BCD_use (self, handle, desc, num, code);
-	if (!use_bcd) {
-		g_free (toc);
-
-		BRASERO_MEDIA_LOG ("Fallback to formatted toc");
-		return BRASERO_BURN_ERR;
-	}
-
-	if (use_bcd > 0)
-		use_bcd = 0;
-
-	if (use_bcd) {
-		BRASERO_MEDIA_LOG ("Using BCD format");
-	}
-	else {
-		BRASERO_MEDIA_LOG ("Using HEX format");
-	}
-
-	for (i = 0; i < num; i++, desc ++) {
-		BraseroMediumTrack *track;
-
-		track = NULL;
-		if (desc->adr == 1 && desc->point <= BRASERO_SCSI_Q_SUB_CHANNEL_TRACK_START) {
-			track = g_new0 (BraseroMediumTrack, 1);
-			track->session = desc->session_num;
-
-			brasero_medium_set_track_type (self, track, desc->control);
-			if (use_bcd)
-				track->start = BRASERO_MSF_TO_LBA (BRASERO_GET_BCD (desc->p_min),
-								   BRASERO_GET_BCD (desc->p_sec),
-								   BRASERO_GET_BCD (desc->p_frame));
-			else
-				track->start = BRASERO_MSF_TO_LBA (desc->p_min,
-								   desc->p_sec,
-								   desc->p_frame);
-
-			track->start -= 150;
-
-			/* if there are tracks and the last previously added track is in
-			 * the same session then set the size */
-			if (priv->tracks) {
-				BraseroMediumTrack *last_track;
-
-				last_track = priv->tracks->data;
-				if (last_track->session == track->session)
-					last_track->blocks_num = track->start - last_track->start;
-			}
-
-			priv->tracks = g_slist_prepend (priv->tracks, track);
-		}
-		else if (desc->point == BRASERO_SCSI_Q_SUB_CHANNEL_LEADOUT_START) {
-			/* NOTE: the leadout session is first in the list. So if
-			 * we have tracks in the list set the last session track
-			 * size when we reach a new leadout (and therefore a new
-			 * session). */
-
-			if (priv->tracks) {
-				BraseroMediumTrack *last_track;
-
-				last_track = priv->tracks->data;
-				last_track->blocks_num = leadout_start - last_track->start;
-			}
-
-			if (use_bcd)
-				leadout_start = BRASERO_MSF_TO_LBA (BRASERO_GET_BCD (desc->p_min),
-								    BRASERO_GET_BCD (desc->p_sec),
-								    BRASERO_GET_BCD (desc->p_frame));
-			else
-				leadout_start = BRASERO_MSF_TO_LBA (desc->p_min,
-								    desc->p_sec,
-								    desc->p_frame);
-			leadout_start -= 150;
-		}
-	}
-
-	if (priv->tracks) {
-		BraseroMediumTrack *last_track;
-
-		/* set the last found track size */
-		last_track = priv->tracks->data;
-		last_track->blocks_num = leadout_start - last_track->start;
-	}
-
-	/* Add a leadout */
-	if (!(priv->info & BRASERO_MEDIUM_CLOSED)) {
-		BraseroMediumTrack *track;
-
-		/* we shouldn't request info on leadout if the disc is closed */
-		track = g_new0 (BraseroMediumTrack, 1);
-		priv->tracks = g_slist_prepend (priv->tracks, track);
-		track->start = leadout_start;
-		track->type = BRASERO_MEDIUM_TRACK_LEADOUT;
-
-		brasero_medium_track_get_info (self,
-					       FALSE,
-					       track,
-					       g_slist_length (priv->tracks),
-					       handle,
-					       code); 
-	}
-
-	priv->tracks = g_slist_reverse (priv->tracks);
-
-	for (iter = priv->tracks; iter; iter = iter->next) {
-		BraseroMediumTrack *track;
-
-		track = iter->data;
-
-		/* check for tracks less that 300 sectors */
-		if (track->blocks_num <= 300 && track->type != BRASERO_MEDIUM_TRACK_LEADOUT) {
-			BRASERO_MEDIA_LOG ("300 sectors size. Checking for real size");
-			brasero_medium_track_volume_size (self, track, handle);
-		}
-
-		BRASERO_MEDIA_LOG ("Track %i: type = %i start = %llu size = %llu",
-				  g_slist_index (priv->tracks, track),
-				  track->type,
-				  track->start,
-				  track->blocks_num);
-	}
-
-	g_free (toc);
-	return BRASERO_BURN_OK;
-}
-
-static BraseroBurnResult
-brasero_medium_old_drive_get_disc_info (BraseroMedium *self,
-					BraseroDeviceHandle *handle,
-					BraseroScsiErrCode *code)
-{
-	int size;
-	BraseroScsiResult result;
-	BraseroMediumPrivate *priv;
-	BraseroScsiDiscInfoStd *info = NULL;
-
-	BRASERO_MEDIA_LOG ("Retrieving media status for old drive");
-
-	priv = BRASERO_MEDIUM_PRIVATE (self);
-
-	result = brasero_mmc1_read_disc_information_std (handle,
-							 &info,
-							 &size,
-							 code);
-	if (result != BRASERO_SCSI_OK) {
-		g_free (info);
-	
-		BRASERO_MEDIA_LOG ("READ DISC INFORMATION failed for old drive");
-		return BRASERO_BURN_ERR;
-	}
-
-	/* Try to identify the type: can only be CDROM CDR CDRW.
-	 * NOTE: since there is no way to distinguish a CDROM and a closed CDR 
-	 * if the disc is closed we set it as CDROM (except if it's RW). */
-	if (info->erasable)
-		priv->info = BRASERO_MEDIUM_CDRW;
-	else if (info->status == BRASERO_SCSI_DISC_FINALIZED)
-		priv->info = BRASERO_MEDIUM_CDROM;
-	else
-		priv->info = BRASERO_MEDIUM_CDR;
-
-	if (info->status == BRASERO_SCSI_DISC_EMPTY) {
-		priv->info |= BRASERO_MEDIUM_BLANK;
-		priv->block_size = 2048;
-		priv->next_wr_add = 0;
-		BRASERO_MEDIA_LOG ("Empty media (old drive)");
-	}
-	else if (info->status == BRASERO_SCSI_DISC_INCOMPLETE) {
-		priv->info |= BRASERO_MEDIUM_APPENDABLE;
-		priv->block_size = 2048;
-		priv->next_wr_add = 0;
-		BRASERO_MEDIA_LOG ("Appendable media (old drive)");
-	}
-	else if (info->status == BRASERO_SCSI_DISC_FINALIZED) {
-		priv->info |= BRASERO_MEDIUM_CLOSED;
-		BRASERO_MEDIA_LOG ("Closed media (old drive)");
-	}
-
-	/* get the contents */
-	result = brasero_medium_get_CD_sessions_info (self, handle, code);
-	return result;
-}
-
-static BraseroBurnResult
-brasero_medium_check_old_drive (BraseroMedium *self,
-				BraseroDeviceHandle *handle,
-				BraseroScsiErrCode *code)
-{
-	gchar *model;
-	BraseroMediumPrivate *priv;
-
-	priv = BRASERO_MEDIUM_PRIVATE (self);
-
-	model = brasero_drive_get_display_name (priv->drive);
-	if (!model)
-		return BRASERO_BURN_ERR;
-
-	if (!strcmp (model, "CD-R55S")) {
-		g_free (model);
-		priv->max_rd = BRASERO_SPEED_TO_RATE_CD (12);
-		priv->max_wrt = BRASERO_SPEED_TO_RATE_CD (4);
-		return brasero_medium_old_drive_get_disc_info (self,
-							       handle,
-							       code);
-	}
-
-	BRASERO_MEDIA_LOG ("Not an old drive model");
-
-	return BRASERO_BURN_ERR;
-}
-
 /**
  * Some identification functions
  */
@@ -2615,19 +2222,15 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 				BraseroDeviceHandle *handle,
 				BraseroScsiErrCode *code)
 {
-	BraseroScsiGetConfigHdr *hdr = NULL;
+	BraseroScsiProfile profile;
 	BraseroMediumPrivate *priv;
 	BraseroScsiResult result;
-	int size;
 
 	BRASERO_MEDIA_LOG ("Retrieving media profile");
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
-	result = brasero_mmc2_get_configuration_feature (handle,
-							 BRASERO_SCSI_FEAT_REAL_TIME_STREAM,
-							 &hdr,
-							 &size,
-							 code);
+	result = brasero_mmc2_get_profile (handle, &profile, code);
+
 	if (result != BRASERO_SCSI_OK) {
 		BraseroScsiAtipData *data = NULL;
 		int size = 0;
@@ -2640,16 +2243,14 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 		 * will be provided by read_disc_information. */
 
 		/* retrieve the speed */
-		result = brasero_medium_get_page_2A_max_speed (self,
-							       handle,
-							       code);
+		result = brasero_medium_get_page_2A_write_speed_desc (self,
+								      handle,
+								      code);
 
 		/* If this fails it means that this drive is probably older than
-		 * MMC1 spec or does not conform to it. Try our last chance. */
+		 * MMC1 spec or does not conform to it. */
 		if (result != BRASERO_BURN_OK)
-			return brasero_medium_check_old_drive (self,
-							       handle,
-							       code);
+			return BRASERO_BURN_ERR;
 
 		/* The only thing here left to determine is if that's a WRITABLE
 		 * or a REWRITABLE. To determine that information, we need to
@@ -2692,7 +2293,7 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 		return result;
 	}
 
-	switch (BRASERO_GET_16 (hdr->current_profile)) {
+	switch (profile) {
 	case BRASERO_SCSI_PROF_CDROM:
 		priv->info = BRASERO_MEDIUM_CDROM;
 		priv->type = types [1];
@@ -2797,43 +2398,15 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 	case BRASERO_SCSI_PROF_HD_DVD_R:
 	case BRASERO_SCSI_PROF_HD_DVD_RAM:
 		priv->info = BRASERO_MEDIUM_UNSUPPORTED;
-		g_free (hdr);
 		return BRASERO_BURN_NOT_SUPPORTED;
 	}
 
-	/* try all SCSI functions to get write/read speeds in order */
-	if (hdr->desc->add_len >= sizeof (BraseroScsiRTStreamDesc)) {
-		BraseroScsiRTStreamDesc *stream;
+	/* Get a more precise idea of what sequential BD-R type we have here */
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BDR_SRM)) {
+		BraseroScsiGetConfigHdr *hdr = NULL;
+		int size = 0;
 
-		/* means it's at least an MMC3 drive */
-		stream = (BraseroScsiRTStreamDesc *) hdr->desc->data;
-		if (stream->wrt_spd) {
-			result = brasero_medium_get_speed_mmc3 (self, handle, code);
-			if (result == BRASERO_BURN_OK)
-				goto end;
-		}
-
-		if (stream->mp2a) {
-			result = brasero_medium_get_page_2A_write_speed_desc (self, handle, code);
-			if (result == BRASERO_BURN_OK)
-				goto end;
-		}
-	}
-
-	/* fallback for speeds */
-	result = brasero_medium_get_page_2A_max_speed (self, handle, code);
-
-end:
-
-	g_free (hdr);
-
-	if (result != BRASERO_BURN_OK)
-		return result;
-
-	/* for BDs media we need to check the number of layers */
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BD|BRASERO_MEDIUM_SRM)) {
-		/* check for POW */
-		hdr = NULL;
+		/* check for POW type */
 		result = brasero_mmc2_get_configuration_feature (handle,
 								 BRASERO_SCSI_FEAT_BDR_POW,
 								 &hdr,
@@ -2873,7 +2446,7 @@ end:
 	}
 
 	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BD)) {
-		/* check for dual layer BD */
+		/* FIXME: check for dual layer BD */
 	}
 
 	return BRASERO_BURN_OK;
@@ -3215,7 +2788,16 @@ brasero_medium_init_real (BraseroMedium *object,
 	if (priv->probe_cancelled)
 		return;
 
+	result = brasero_medium_get_speed (object, handle, &code);
+	if (result != BRASERO_BURN_OK)
+		return;
+
+	if (priv->probe_cancelled)
+		return;
+
 	brasero_medium_get_capacity_by_type (object, handle, &code);
+	if (priv->probe_cancelled)
+		return;
 
 	result = brasero_medium_get_contents (object, handle, &code);
 	if (result != BRASERO_BURN_OK)
@@ -3233,7 +2815,6 @@ brasero_medium_init_real (BraseroMedium *object,
 		return;
 
 	brasero_medium_init_caps (object, handle, &code);
-
 	if (priv->probe_cancelled)
 		return;
 
@@ -3274,9 +2855,11 @@ brasero_medium_probed (gpointer data)
 	priv = BRASERO_MEDIUM_PRIVATE (data);
 
 	/* This signal must be emitted in the main thread */
+	GDK_THREADS_ENTER ();
 	g_signal_emit (data,
 		       medium_signals [PROBED],
 		       0);
+	GDK_THREADS_LEAVE ();
 
 	priv->probe_id = 0;
 	return FALSE;
@@ -3530,12 +3113,15 @@ brasero_medium_class_init (BraseroMediumClass *klass)
  *
  **/
 gboolean
-brasero_medium_can_be_written (BraseroMedium *self)
+brasero_medium_can_be_written (BraseroMedium *medium)
 {
 	BraseroMediumPrivate *priv;
 	BraseroDriveCaps caps;
 
-	priv = BRASERO_MEDIUM_PRIVATE (self);
+	g_return_val_if_fail (medium != NULL, FALSE);
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), FALSE);
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
 	if (!(priv->info & BRASERO_MEDIUM_REWRITABLE)
 	&&   (priv->info & BRASERO_MEDIUM_CLOSED))
@@ -3586,12 +3172,15 @@ brasero_medium_can_be_written (BraseroMedium *self)
  *
  **/
 gboolean
-brasero_medium_can_be_rewritten (BraseroMedium *self)
+brasero_medium_can_be_rewritten (BraseroMedium *medium)
 {
 	BraseroMediumPrivate *priv;
 	BraseroDriveCaps caps;
 
-	priv = BRASERO_MEDIUM_PRIVATE (self);
+	g_return_val_if_fail (medium != NULL, FALSE);
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), FALSE);
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
 	if (!(priv->info & BRASERO_MEDIUM_REWRITABLE)
 	||   (priv->info & BRASERO_MEDIUM_FILE))
@@ -3626,14 +3215,16 @@ brasero_medium_can_be_rewritten (BraseroMedium *self)
  *
  **/
 BraseroDrive *
-brasero_medium_get_drive (BraseroMedium *self)
+brasero_medium_get_drive (BraseroMedium *medium)
 {
 	BraseroMediumPrivate *priv;
 
-	if (!self)
+	if (!medium)
 		return NULL;
 
-	priv = BRASERO_MEDIUM_PRIVATE (self);
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), NULL);
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
 	return priv->drive;
 }
 
@@ -3647,12 +3238,15 @@ brasero_medium_get_drive (BraseroMedium *self)
  *
  **/
 const gchar *
-brasero_medium_get_CD_TEXT_title (BraseroMedium *self)
+brasero_medium_get_CD_TEXT_title (BraseroMedium *medium)
 {
 	BraseroMediumPrivate *priv;
 
-	priv = BRASERO_MEDIUM_PRIVATE (self);
-	return priv->CD_TEXT_title;;
+	g_return_val_if_fail (medium != NULL, NULL);
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (medium), NULL);
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
+	return priv->CD_TEXT_title;
 
 }
 
