@@ -170,8 +170,6 @@ struct BraseroProjectManagerPrivate {
 	guint status_ctx;
 
 	GtkActionGroup *action_group;
-
-	guint oneshot:1;
 };
 
 #define BRASERO_PROJECT_MANAGER_CONNECT_CHANGED(manager, container)		\
@@ -526,12 +524,6 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 	||  manager->priv->type == BRASERO_PROJECT_TYPE_VIDEO) {
 		if (!brasero_project_confirm_switch (BRASERO_PROJECT (manager->priv->project)))
 			return;
-
-		if (manager->priv->oneshot) {
-			/* Here we may have to close brasero altogether */
-			gtk_widget_destroy (toplevel);
-			return;
-		}
 	}
 
 	if (manager->priv->status_ctx) {
@@ -544,7 +536,6 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 	action = gtk_action_group_get_action (manager->priv->action_group, "NewChoose");
 
 	manager->priv->type = type;
-
 	if (type == BRASERO_PROJECT_TYPE_INVALID) {
 		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_NONE);
 		brasero_project_set_none (BRASERO_PROJECT (manager->priv->project));
@@ -607,12 +598,12 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 		if (toplevel)
 			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - New Image File"));
 
-		brasero_app_burn_image (brasero_app_get_default (), uri);
 		brasero_project_manager_switch (manager,
 						BRASERO_PROJECT_TYPE_INVALID,
 						NULL,
 						NULL,
 						TRUE);
+		brasero_app_burn_image (brasero_app_get_default (), uri);
 	}
 	else if (type == BRASERO_PROJECT_TYPE_COPY) {
 		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_NONE);
@@ -624,12 +615,12 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 		if (toplevel)
 			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - Disc Copy"));
 
-		brasero_app_copy_disc (brasero_app_get_default (), uri, NULL);
 		brasero_project_manager_switch (manager,
 						BRASERO_PROJECT_TYPE_INVALID,
 						NULL,
 						NULL,
 						TRUE);
+		brasero_app_copy_disc (brasero_app_get_default (), uri, NULL);
 	}
 }
 
@@ -638,66 +629,49 @@ brasero_project_manager_type_changed_cb (BraseroProjectTypeChooser *chooser,
 					 BraseroProjectType type,
 					 BraseroProjectManager *manager)
 {
-	manager->priv->oneshot = FALSE;
 	brasero_project_manager_switch (manager, type, NULL, NULL, TRUE);
 }
 
 static void
 brasero_project_manager_new_empty_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
-	manager->priv->oneshot = FALSE;
 	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL, TRUE);
 }
 
 static void
 brasero_project_manager_new_audio_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
-	manager->priv->oneshot = FALSE;
 	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_AUDIO, NULL, NULL, TRUE);
 }
 
 static void
 brasero_project_manager_new_data_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
-	manager->priv->oneshot = FALSE;
 	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_DATA, NULL, NULL, TRUE);
 }
 
 static void
 brasero_project_manager_new_video_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
-	manager->priv->oneshot = FALSE;
 	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_VIDEO, NULL, NULL, TRUE);
 }
 
 static void
 brasero_project_manager_new_copy_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
-	manager->priv->oneshot = FALSE;
 	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_COPY, NULL, NULL, TRUE);
 }
 
 static void
 brasero_project_manager_new_iso_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
-	manager->priv->oneshot = FALSE;
 	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_ISO, NULL, NULL, TRUE);
 }
 
-void
-brasero_project_manager_audio (BraseroProjectManager *manager,
-			       GSList *uris)
-{
-	brasero_project_manager_switch (manager,
-					BRASERO_PROJECT_TYPE_AUDIO,
-					uris,
-					NULL,
-					TRUE);
-}
-
 void
 brasero_project_manager_data (BraseroProjectManager *manager,
-			      GSList *uris)
+			      GSList *uris,
+			      gboolean burn)
 {
 	gchar *burn_URI = NULL;
 
@@ -707,9 +681,8 @@ brasero_project_manager_data (BraseroProjectManager *manager,
 		uris = g_slist_prepend (NULL, burn_URI);
 	}
 
-	if (manager->priv->oneshot) {
-		brasero_project_set_data (BRASERO_PROJECT (manager->priv->project),
-					  uris);
+	if (burn) {
+		brasero_project_set_data (BRASERO_PROJECT (manager->priv->project), uris);
 		brasero_project_burn (BRASERO_PROJECT (manager->priv->project));
 	}
 	else
@@ -718,7 +691,7 @@ brasero_project_manager_data (BraseroProjectManager *manager,
 						uris,
 						NULL,
 						TRUE);
-	
+
 	if (burn_URI) {
 		g_slist_free (uris);
 		g_free (burn_URI);
@@ -726,14 +699,37 @@ brasero_project_manager_data (BraseroProjectManager *manager,
 }
 
 void
-brasero_project_manager_video (BraseroProjectManager *manager,
-			       GSList *uris)
+brasero_project_manager_audio (BraseroProjectManager *manager,
+			       GSList *uris,
+			       gboolean burn)
 {
-	brasero_project_manager_switch (manager,
-					BRASERO_PROJECT_TYPE_VIDEO,
-					uris,
-					NULL,
-					TRUE);
+	if (burn) {
+		brasero_project_set_audio (BRASERO_PROJECT (manager->priv->project), uris);
+		brasero_project_burn (BRASERO_PROJECT (manager->priv->project));
+	}
+	else
+		brasero_project_manager_switch (manager,
+						BRASERO_PROJECT_TYPE_AUDIO,
+						uris,
+						NULL,
+						TRUE);
+}
+
+void
+brasero_project_manager_video (BraseroProjectManager *manager,
+			       GSList *uris,
+			       gboolean burn)
+{
+	if (burn) {
+		brasero_project_set_video (BRASERO_PROJECT (manager->priv->project), uris);
+		brasero_project_burn (BRASERO_PROJECT (manager->priv->project));
+	}
+	else
+		brasero_project_manager_switch (manager,
+						BRASERO_PROJECT_TYPE_VIDEO,
+						uris,
+						NULL,
+						TRUE);
 }
 
 void
@@ -761,75 +757,34 @@ brasero_project_manager_iso (BraseroProjectManager *manager,
 
 BraseroProjectType
 brasero_project_manager_open_project (BraseroProjectManager *manager,
-				      const gchar *uri)
+				      BraseroDiscTrack *track,
+				      const gchar *uri,
+				      gboolean burn)
 {
-	BraseroProjectType type;
 	GtkAction *action;
+	BraseroProjectType type;
 
-	gtk_widget_show (manager->priv->layout);
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (manager), 1);
-	type = brasero_project_open_project (BRASERO_PROJECT (manager->priv->project), uri);
-
-	manager->priv->type = type;
-    	if (type == BRASERO_PROJECT_TYPE_INVALID) {
+    	if (track->type == BRASERO_PROJECT_TYPE_INVALID) {
 		brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL, TRUE);
 		return BRASERO_PROJECT_TYPE_INVALID;
 	}
 
-	if (type == BRASERO_PROJECT_TYPE_DATA)
-		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_DATA);
-	else
-		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_AUDIO);
-
-	action = gtk_action_group_get_action (manager->priv->action_group, "NewChoose");
-	gtk_action_set_sensitive (action, TRUE);
-
-	return type;
-}
-
-void
-brasero_project_manager_burn_project (BraseroProjectManager *manager,
-				      const gchar *uri)
-{
-	BraseroProjectType type;
-
-	brasero_project_manager_set_oneshot (manager, TRUE);
-	type = brasero_project_open_project (BRASERO_PROJECT (manager->priv->project), uri);
+	brasero_project_manager_switch (manager, track->type, NULL, NULL, FALSE);
+	type = brasero_project_open_project (BRASERO_PROJECT (manager->priv->project), track, uri);
 	if (type == BRASERO_PROJECT_TYPE_INVALID)
-		return;
+		return type;
 
-	manager->priv->type = type;
-	brasero_project_burn (BRASERO_PROJECT (manager->priv->project));
-}
-
-#ifdef BUILD_PLAYLIST
-
-BraseroProjectType
-brasero_project_manager_open_playlist (BraseroProjectManager *manager,
-				       const gchar *uri)
-{
-	BraseroProjectType type;
-	GtkAction *action;
-
-    	gtk_widget_show (manager->priv->layout);
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (manager), 1);
-	type = brasero_project_open_playlist (BRASERO_PROJECT (manager->priv->project), uri);
-	manager->priv->type = type;
-
-    	if (type == BRASERO_PROJECT_TYPE_INVALID) {
-		brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL, TRUE);
-		return BRASERO_PROJECT_TYPE_INVALID;
+	if (burn) {
+		brasero_project_burn (BRASERO_PROJECT (manager->priv->project));
+		return track->type;
 	}
 
-	brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_AUDIO);
 	action = gtk_action_group_get_action (manager->priv->action_group, "NewChoose");
 	gtk_action_set_sensitive (action, TRUE);
 
-	return BRASERO_PROJECT_TYPE_AUDIO;
+	return track->type;
 }
 
-#endif
-
 BraseroProjectType
 brasero_project_manager_open_by_mime (BraseroProjectManager *manager,
 				      const gchar *uri,
@@ -838,16 +793,28 @@ brasero_project_manager_open_by_mime (BraseroProjectManager *manager,
 	/* When our files/description of x-brasero mime type is not properly 
 	 * installed, it's returned as application/xml, so check that too. */
 	if (!strcmp (mime, "application/x-brasero")
-	||  !strcmp (mime, "application/xml"))
-		return brasero_project_manager_open_project (manager, uri);
+	||  !strcmp (mime, "application/xml")) {
+		BraseroDiscTrack *track = NULL;
+
+		if (!brasero_project_open_project_xml (uri, &track, TRUE))
+			return BRASERO_PROJECT_TYPE_INVALID;
+
+		return brasero_project_manager_open_project (manager, track, uri, FALSE);
+	}
 
 #ifdef BUILD_PLAYLIST
 
 	else if (!strcmp (mime, "audio/x-scpls")
 	     ||  !strcmp (mime, "audio/x-ms-asx")
 	     ||  !strcmp (mime, "audio/x-mp3-playlist")
-	     ||  !strcmp (mime, "audio/x-mpegurl"))
-		return brasero_project_manager_open_playlist (manager, uri);
+	     ||  !strcmp (mime, "audio/x-mpegurl")) {
+		BraseroDiscTrack *track = NULL;
+
+		if (!brasero_project_open_audio_playlist_project (uri, &track, TRUE))
+			return BRASERO_PROJECT_TYPE_INVALID;
+
+		return brasero_project_manager_open_project (manager, track, uri, FALSE);
+	}
 #endif
 
 	else if (!strcmp (mime, "application/x-cd-image")
@@ -873,7 +840,7 @@ brasero_project_manager_open_uri (BraseroProjectManager *manager,
 	/* FIXME: make that asynchronous */
 	/* NOTE: don't follow symlink because we want to identify them */
 	file = g_file_new_for_commandline_arg (uri_arg);
-	if (file)
+	if (!file)
 		return BRASERO_PROJECT_TYPE_INVALID;
 
 	info = g_file_query_info (file,
@@ -966,7 +933,6 @@ brasero_project_manager_open_cb (GtkAction *action, BraseroProjectManager *manag
 	gint answer;
 	GtkWidget *chooser;
 	GtkWidget *toplevel;
-	BraseroProjectType type;
 
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
 	chooser = gtk_file_chooser_dialog_new (_("Open Project"),
@@ -989,8 +955,7 @@ brasero_project_manager_open_cb (GtkAction *action, BraseroProjectManager *manag
 	uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (chooser));
 	gtk_widget_destroy (chooser);
 
-	manager->priv->oneshot = FALSE;
-	type = brasero_project_manager_open_uri (manager, uri);
+	brasero_project_manager_open_uri (manager, uri);
 	g_free (uri);
 }
 
@@ -999,17 +964,9 @@ brasero_project_manager_recent_clicked_cb (BraseroProjectTypeChooser *chooser,
 					   const gchar *uri,
 					   BraseroProjectManager *manager)
 {
-	manager->priv->oneshot = FALSE;
 	brasero_project_manager_open_uri (manager, uri);
 }
 
-void
-brasero_project_manager_set_oneshot (BraseroProjectManager *manager,
-				     gboolean oneshot)
-{
-	manager->priv->oneshot = oneshot;
-}
-
 void
 brasero_project_manager_empty (BraseroProjectManager *manager)
 {
